/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jaxr;

import java.net.PasswordAuthentication;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.JAXRException;
import javax.xml.registry.JAXRResponse;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.RegistryService;
import javax.xml.registry.infomodel.Classification;
import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.ExternalLink;
import javax.xml.registry.infomodel.InternationalString;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;
import javax.xml.registry.infomodel.SpecificationLink;

public class JAXRFacade {
/*
	public static void setJAXRConnectionFactory(Properties properties) {
		String connectionFactoryClass = (String)properties.get("javax.xml.registry.ConnectionFactoryClass");
		if (connectionFactoryClass == null)
			connectionFactoryClass = "org.apache.ws.scout.registry.ConnectionFactoryImpl";
		System.setProperty("javax.xml.registry.ConnectionFactoryClass", connectionFactoryClass);
	}
	
	public static Connection openJAXRConnection(Properties properties) throws JAXRException {
		
        
	}
	
	
	
	public static Organization getOrganizationByName(Connection jaxrConnection,
													String organizationName) throws JAXRException{
		RegistryService registryService = jaxrConnection.getRegistryService();
		BusinessQueryManager bqm = registryService.getBusinessQueryManager();

		Collection<String> findQualifiers = new ArrayList<String>();
		findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
		Collection<String> namePatterns = new ArrayList<String>();
		namePatterns.add(organizationName);

		BulkResponse response =
		bqm.findOrganizations(findQualifiers, namePatterns, null,
		null, null, null);
		Collection orgs = response.getCollection();
		if (orgs.size() > 1){
			throw new JAXRException("Found more than one result for this organization name");
		}else{
			Organization org = (Organization)orgs.toArray()[0];
			return org;
		}
	}
	
	public static Service getServiceByName(
			Connection jaxrConnection,
			Key organizationKey,
			String serviceName) throws JAXRException{
			RegistryService registryService = jaxrConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();

			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_ASC);
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(serviceName);

			BulkResponse response = bqm.findServices(organizationKey,findQualifiers,namePatterns, null,null);
			Collection orgs = response.getCollection();
			if (orgs.size() > 1){
				throw new JAXRException("Found more than one result for this organization and this serviceName");
			}else{
				Service service = (Service)orgs.toArray()[0];
				return service;
			}
	}
	public static ClassificationScheme getClassificationSchemeByName(Connection jaxrConnection, String schemeName)  throws JAXRException{
		RegistryService registryService = jaxrConnection.getRegistryService();
	    BusinessQueryManager bqm = registryService.getBusinessQueryManager();
		
	    ClassificationScheme scheme = bqm.findClassificationSchemeByName(null, schemeName);
		
	    return scheme;
	}
	public static Key createOrganization(Connection jaxrConnection, 
												  String organizationName,
												  String organizationDescription) throws JAXRException{
		
        RegistryService registryService = jaxrConnection.getRegistryService();
        BusinessLifeCycleManager lifeCycleManager = registryService.getBusinessLifeCycleManager();
        Organization organization = lifeCycleManager.createOrganization(organizationName);
        organization.setDescription(lifeCycleManager.createInternationalString());
        return publishOrganization(jaxrConnection, organization);
        
        
	}
	
	public static void deleteOrganization(Connection jaxrConnection, Key organizationKey) throws JAXRException{
		
		RegistryService registryService = jaxrConnection.getRegistryService();
        BusinessLifeCycleManager lifeCycleManager = registryService.getBusinessLifeCycleManager();
		String id = organizationKey.getId();
        System.out.println("Deleting organization with id " + id);

        Collection<Key> keys = new ArrayList<Key>();
        keys.add(organizationKey);

        BulkResponse response = lifeCycleManager.deleteOrganizations(keys);
        Collection exceptions = response.getExceptions();

        if (exceptions == null) {
            System.out.println("Organization deleted");

            Collection retKeys = response.getCollection();
            for (Object k : retKeys) {
                Key orgKey = (Key) k;
                id = orgKey.getId();
                System.out.println("Organization key was " + id);
            }
        } else {
            for (Object e : exceptions) {
                Exception exception = (Exception) e;
                System.err.println("Exception on delete: " +
                    exception.toString());
            }
        }
	}
	
	
	public static void createServiceForOrganization(Connection jaxrConnection, 
											Key organizationKey,
											String serviceName, 
											String serviceDescription,
											String serviceURI,
											String wsdlLocation) throws JAXRException {
		
		RegistryService registryService = jaxrConnection.getRegistryService();
        BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
        BusinessQueryManager bqm = registryService.getBusinessQueryManager();
        Organization organization = (Organization)bqm.getRegistryObject(organizationKey.getId(), LifeCycleManager.ORGANIZATION);
        //Create services and service
        Collection<Service> services = new ArrayList<Service>();
        InternationalString iServiceName = blcm.createInternationalString(serviceName);
        Service service = blcm.createService(iServiceName);
        InternationalString iServiceDescription = blcm.createInternationalString(serviceDescription);
        service.setDescription(iServiceDescription);
        //	 Create service bindings
        Collection<ServiceBinding> serviceBindings = new ArrayList<ServiceBinding>();
        ServiceBinding binding = blcm.createServiceBinding();
        InternationalString iServiceBindingDescritpion = blcm.createInternationalString("Binding For " + serviceName );
        binding.setDescription(iServiceBindingDescritpion);
        //	 allow us to publish a fictitious URI without an error
        binding.setValidateURI(false);
        binding.setAccessURI(serviceURI);
        
	
        
        
        if (wsdlLocation != null){
        		Concept specConcept =
        		blcm.createConcept(null, blcm.createInternationalString("Concept for service "+ serviceName), "");
        		InternationalString s = blcm.createInternationalString("Concept for service "+ serviceName);
        		specConcept.setDescription(s);
        		ExternalLink wsdlLink =
        		blcm.createExternalLink(wsdlLocation,serviceName + "WSDL");
        		specConcept.addExternalLink(wsdlLink);
        		
        		// Classifying the concept as WSDL
        		// String uuid_types = "uuid:c1acf26d-9672-4404-9d70-39b756e62ab4";
        		String schemeName = "uddi-org:types";
        		ClassificationScheme uddiOrgTypes =
        			(ClassificationScheme) bqm.findClassificationSchemeByName(null, schemeName);
        			
        		Classification wsdlSpecClassification =
        			blcm.createClassification(uddiOrgTypes,
        			"wsdlSpec", "wsdlSpec");
        		specConcept.addClassification(wsdlSpecClassification);
        		
        		
        		Collection<Concept> concepts = new ArrayList<Concept>();
        		concepts.add(specConcept);
        		BulkResponse concResponse = blcm.saveConcepts(concepts);
        		
        		String conceptKeyId = null;
        		Collection concExceptions = concResponse.getExceptions();
        		Key concKey = null;
        		if (concExceptions == null) {
        			System.out.println("WSDL Specification Concept saved");
        			Collection keys = concResponse.getCollection();
        			Iterator<Key> keyIter = keys.iterator();
        			if (keyIter.hasNext()) {
        				concKey = (Key) keyIter.next();
        				conceptKeyId = concKey.getId();
        				System.out.println("Concept key is " + conceptKeyId);
        			}
        		}
        		
        		
        		specConcept = (Concept) bqm.getRegistryObject(conceptKeyId,
        			LifeCycleManager.CONCEPT);
        			
        		SpecificationLink specLink = blcm.createSpecificationLink();
        		specLink.setSpecificationObject(specConcept);
        		binding.addSpecificationLink(specLink);
        }
        serviceBindings.add(binding);
        //	 Add service bindings to service
        service.addServiceBindings(serviceBindings);
        //   Add service to services, then add services to organization
        services.add(service);
        organization.addServices(services);
        
        publishOrganization(jaxrConnection, organization);
        
	}
	
	
	public static Key publishOrganization(Connection jaxrConnection, Organization organization) throws JAXRException{
		
		RegistryService registryService = jaxrConnection.getRegistryService();
        BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
        
		Collection orgs = new ArrayList();
        orgs.add(organization);

        //Save the Organization in the UDDI registry.
        BulkResponse bulkResponse = blcm.saveOrganizations(orgs);

        //Obtain the Organization's Key (the UDDI businessEntity's businessKey) from the response.
        if (bulkResponse.getExceptions() == null)
        {
        	//1 Organization was saved, so 1 key will be returned in the response collection
        	Collection responses = bulkResponse.getCollection();
        	Key organizationKey = (Key)responses.iterator().next();
        	System.out.println("\nOrganization Key = " + organizationKey.getId() + "Published");
        	return organizationKey;
        }else{
        	return null;
        }
	
	}
	
	public static Key createClassificationScheme(Connection jaxrConnection, 
												String classificationSchemeName,
												String classificationSchemeDescription,
												List<String> values) throws JAXRException{
		
		RegistryService registryService = jaxrConnection.getRegistryService();
    	BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
    	BusinessQueryManager bqm = registryService.getBusinessQueryManager();
    
		InternationalString sn = blcm.createInternationalString(classificationSchemeName);
		InternationalString sd = blcm.createInternationalString(classificationSchemeDescription);
		ClassificationScheme myScheme =blcm.createClassificationScheme(sn, sd);
		String schemeName = "uddi-org:types";
		ClassificationScheme uddiOrgTypes =
			(ClassificationScheme) bqm.findClassificationSchemeByName(null, schemeName);
		
		if (uddiOrgTypes != null) {
			
			Iterator<String> it = values.iterator();
			while(it.hasNext()){
				String value = it.next();
				InternationalString cn = blcm.createInternationalString(value);
                Classification classification = blcm.createClassification(uddiOrgTypes, cn, value);
                myScheme.addClassification(classification);
			}
			
			InternationalString ld = blcm.createInternationalString(classificationSchemeName);
			
			Collection<ClassificationScheme> schemes = new ArrayList<ClassificationScheme>();
			schemes.add(myScheme);
			BulkResponse br = blcm.saveClassificationSchemes(schemes);
			
			if (br.getStatus() == JAXRResponse.STATUS_SUCCESS) {
				System.out.println("Saved ClassificationScheme");
				Collection schemeKeys = br.getCollection();
				Iterator keysIter = schemeKeys.iterator();
				Key key = null;
				while (keysIter.hasNext()) {
					key = (Key) keysIter.next();
					System.out.println("mySchemeId key is " +key.getId());
					System.out.println("Use this key as the scheme" + " uuid in the taxonomy file");
				} 
				return key;
			} else {
				return null;
			}
		}else{
			return null;
		}	

	}
	
	public static void classifyService(Connection jaxrConnection, Organization organization, Service service, ClassificationScheme cScheme, String classificationValue) throws JAXRException {
		RegistryService registryService = jaxrConnection.getRegistryService();
    	BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
    	BusinessQueryManager bqm = registryService.getBusinessQueryManager();
    	
		Classification classification = blcm.createClassification(cScheme, blcm.createInternationalString(classificationValue), classificationValue);
		service.addClassification(classification);
		publishOrganization(jaxrConnection, organization);
	}
	*/
}

